@@ -44,7 +44,7 @@ struct _IdeDebugManager
 
   GHashTable     *breakpoints;
   IdeDebugger    *debugger;
-  IdeSignalGroup *debugger_signals;
+  GSignalGroup   *debugger_signals;
   GQueue          pending_breakpoints;
   GPtrArray      *supported_languages;
 
@@ -237,7 +237,7 @@ ide_debug_manager_load_supported_languages (IdeDebugManager *self)
                            self,
                            G_CONNECT_SWAPPED);
 
-  loaded_plugins = peas_engine_get_loaded_plugins (engine);
+  loaded_plugins = peas_engine_dup_loaded_plugins (engine);
 
   for (guint i = 0; loaded_plugins[i] != NULL; i++)
     {
@@ -529,7 +529,7 @@ ide_debug_manager_real_breakpoint_reached (IdeDebugManager       *self,
 }
 
 static void
-ide_debug_manager_dispose (GObject *object)
+ide_debug_manager_destroy (IdeObject *object)
 {
   IdeDebugManager *self = (IdeDebugManager *)object;
 
@@ -537,10 +537,10 @@ ide_debug_manager_dispose (GObject *object)
   g_queue_clear (&self->pending_breakpoints);
 
   g_hash_table_remove_all (self->breakpoints);
-  ide_signal_group_set_target (self->debugger_signals, NULL);
+  g_signal_group_set_target (self->debugger_signals, NULL);
   ide_clear_and_destroy_object (&self->debugger);
 
-  G_OBJECT_CLASS (ide_debug_manager_parent_class)->dispose (object);
+  IDE_OBJECT_CLASS (ide_debug_manager_parent_class)->destroy (object);
 }
 
 static void
@@ -600,10 +600,10 @@ ide_debug_manager_class_init (IdeDebugManagerClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   IdeObjectClass *i_object_class = IDE_OBJECT_CLASS (klass);
 
-  object_class->dispose = ide_debug_manager_dispose;
   object_class->finalize = ide_debug_manager_finalize;
   object_class->get_property = ide_debug_manager_get_property;
 
+  i_object_class->destroy = ide_debug_manager_destroy;
   i_object_class->parent_set = ide_debug_manager_parent_set;
 
   /**
@@ -699,29 +699,29 @@ ide_debug_manager_init (IdeDebugManager *self)
                                              g_object_unref,
                                              g_object_unref);
 
-  self->debugger_signals = ide_signal_group_new (IDE_TYPE_DEBUGGER);
+  self->debugger_signals = g_signal_group_new (IDE_TYPE_DEBUGGER);
 
-  ide_signal_group_connect_swapped (self->debugger_signals,
+  g_signal_group_connect_swapped (self->debugger_signals,
                                     "stopped",
                                     G_CALLBACK (ide_debug_manager_debugger_stopped),
                                     self);
 
-  ide_signal_group_connect_swapped (self->debugger_signals,
+  g_signal_group_connect_swapped (self->debugger_signals,
                                     "running",
                                     G_CALLBACK (ide_debug_manager_debugger_running),
                                     self);
 
-  ide_signal_group_connect_swapped (self->debugger_signals,
+  g_signal_group_connect_swapped (self->debugger_signals,
                                     "breakpoint-added",
                                     G_CALLBACK (ide_debug_manager_breakpoint_added),
                                     self);
 
-  ide_signal_group_connect_swapped (self->debugger_signals,
+  g_signal_group_connect_swapped (self->debugger_signals,
                                     "breakpoint-modified",
                                     G_CALLBACK (ide_debug_manager_breakpoint_modified),
                                     self);
 
-  ide_signal_group_connect_swapped (self->debugger_signals,
+  g_signal_group_connect_swapped (self->debugger_signals,
                                     "breakpoint-removed",
                                     G_CALLBACK (ide_debug_manager_breakpoint_removed),
                                     self);
@@ -730,7 +730,7 @@ ide_debug_manager_init (IdeDebugManager *self)
 static void
 debugger_lookup (PeasExtensionSet *set,
                  PeasPluginInfo   *plugin_info,
-                 PeasExtension    *exten,
+                 GObject    *exten,
                  gpointer          user_data)
 {
   DebuggerLookup *lookup = user_data;
@@ -986,7 +986,7 @@ _ide_debug_manager_prepare (IdeDebugManager  *self,
     ide_run_context_setenv (run_context, "G_DEBUG", "fatal-criticals");
 
   if (g_set_object (&self->debugger, debugger))
-    ide_signal_group_set_target (self->debugger_signals, self->debugger);
+    g_signal_group_set_target (self->debugger_signals, self->debugger);
 
   ide_debug_manager_set_active (self, TRUE);
 
