@@ -219,7 +219,7 @@ ide_application_startup (GApplication *app)
   styles = gtk_source_style_scheme_manager_get_default ();
   style_path = g_build_filename (g_get_home_dir (), ".local", "share", "gtksourceview-5", "styles", NULL);
   gtk_source_style_scheme_manager_append_search_path (styles, style_path);
-  gtk_source_style_scheme_manager_append_search_path (styles, PACKAGE_DATADIR"/gnome-builder/styles/");
+  gtk_source_style_scheme_manager_append_search_path (styles, PACKAGE_DATADIR"/styles/");
 
   /* Add custom locations for language specs */
   langs = gtk_source_language_manager_get_default ();
@@ -1143,3 +1143,24 @@ ide_application_find_project_workbench (IdeApplication *self,
 
   return NULL;
 }
+
+gboolean
+ide_application_control_is_pressed (IdeApplication *self)
+{
+  GdkDisplay *display;
+  GdkSeat *seat;
+  GdkDevice *keyboard;
+  GdkModifierType modifiers;
+
+  if (self == NULL)
+    self = IDE_APPLICATION_DEFAULT;
+
+  g_return_val_if_fail (IDE_IS_APPLICATION (self), FALSE);
+
+  display = gdk_display_get_default ();
+  seat = gdk_display_get_default_seat (display);
+  keyboard = gdk_seat_get_keyboard (seat);
+  modifiers = gdk_device_get_modifier_state (keyboard) & gtk_accelerator_get_default_mod_mask ();
+
+  return !!(modifiers & GDK_CONTROL_MASK);
+}
