@@ -56,7 +56,7 @@ struct _IdePrimaryWorkspace
   IdeRunButton       *run_button;
   GtkLabel           *project_title;
   GtkMenuButton      *add_button;
-  GtkMenuButton      *sidebar_add_button;
+  GtkMenuButton      *sidebar_menu_button;
   GtkOverlay         *overlay;
   IdeOmniBar         *omni_bar;
   IdeJoinedMenu      *build_menu;
@@ -353,7 +353,7 @@ ide_primary_workspace_class_init (IdePrimaryWorkspaceClass *klass)
   gtk_widget_class_bind_template_child (widget_class, IdePrimaryWorkspace, project_title);
   gtk_widget_class_bind_template_child (widget_class, IdePrimaryWorkspace, run_button);
   gtk_widget_class_bind_template_child (widget_class, IdePrimaryWorkspace, statusbar);
-  gtk_widget_class_bind_template_child (widget_class, IdePrimaryWorkspace, sidebar_add_button);
+  gtk_widget_class_bind_template_child (widget_class, IdePrimaryWorkspace, sidebar_menu_button);
   gtk_widget_class_bind_template_callback (widget_class, _ide_workspace_adopt_widget);
   gtk_widget_class_bind_template_callback (widget_class, invert_boolean);
 
@@ -372,6 +372,7 @@ ide_primary_workspace_class_init (IdePrimaryWorkspaceClass *klass)
 static void
 ide_primary_workspace_init (IdePrimaryWorkspace *self)
 {
+  GtkPopover *popover;
   GMenu *menu;
 
   ide_workspace_set_id (IDE_WORKSPACE (self), "primary");
@@ -380,10 +381,15 @@ ide_primary_workspace_init (IdePrimaryWorkspace *self)
 
   menu = ide_application_get_menu_by_id (IDE_APPLICATION_DEFAULT, "new-document-menu");
   gtk_menu_button_set_menu_model (self->add_button, G_MENU_MODEL (menu));
-  gtk_menu_button_set_menu_model (self->sidebar_add_button, G_MENU_MODEL (menu));
+
+  menu = ide_application_get_menu_by_id (IDE_APPLICATION_DEFAULT, "ide-primary-workspace-menu");
+  gtk_menu_button_set_menu_model (self->sidebar_menu_button, G_MENU_MODEL (menu));
 
   menu = ide_application_get_menu_by_id (IDE_APPLICATION_DEFAULT, "build-menu");
   ide_joined_menu_append_menu (self->build_menu, G_MENU_MODEL (menu));
+
+  popover = gtk_menu_button_get_popover (self->sidebar_menu_button);
+  ide_header_bar_setup_menu (GTK_POPOVER_MENU (popover));
 }
 
 /**
