@@ -25,7 +25,6 @@
 #include <glib/gi18n.h>
 
 #include "ide-search-popover-private.h"
-#include "ide-search-popover-group-private.h"
 #include "ide-search-resources.h"
 #include "ide-gui-global.h"
 
@@ -34,26 +33,17 @@
 
 struct _IdeSearchPopover
 {
-  GtkPopover          parent_instance;
+  AdwDialog           parent_instance;
 
   GCancellable       *cancellable;
   IdeSearchEngine    *search_engine;
   GSettings          *settings;
 
-  GListStore         *groups;
-
-  GtkWidget          *left;
-  GtkWidget          *right;
-  GtkWidget          *center;
-
-  GtkSearchEntry     *entry;
+  GtkText            *text;
   GtkListView        *list_view;
   AdwBin             *preview_bin;
-  AdwWindowTitle     *preview_title;
-  GtkToggleButton    *preview_toggle;
-  GtkRevealer        *preview_revealer;
-  GtkListBox         *providers_list_box;
   GtkSingleSelection *selection;
+  GtkStack           *stack;
 
   IdeSearchCategory   last_category;
 
@@ -72,13 +62,9 @@ enum {
   N_PROPS
 };
 
-static void buildable_iface_init (GtkBuildableIface *iface);
-
-G_DEFINE_FINAL_TYPE_WITH_CODE (IdeSearchPopover, ide_search_popover, GTK_TYPE_POPOVER,
-                               G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE, buildable_iface_init))
+G_DEFINE_FINAL_TYPE (IdeSearchPopover, ide_search_popover, ADW_TYPE_DIALOG)
 
 static GParamSpec *properties [N_PROPS];
-static GtkBuildableIface *parent_buildable_iface;
 
 static void
 ide_search_popover_cancel (IdeSearchPopover *self)
@@ -120,7 +106,7 @@ ide_search_popover_activate (IdeSearchPopover *self,
   if (last_focus == NULL)
     last_focus = GTK_WIDGET (workspace);
 
-  gtk_popover_popdown (GTK_POPOVER (self));
+  adw_dialog_close (ADW_DIALOG (self));
 
   ide_search_result_activate (result, last_focus);
 
@@ -140,77 +126,12 @@ ide_search_popover_hide_action (GtkWidget  *widget,
   workspace = ide_widget_get_workspace (widget);
   page = ide_workspace_get_most_recent_page (workspace);
 
-  gtk_popover_popdown (GTK_POPOVER (widget));
+  adw_dialog_close (ADW_DIALOG (widget));
 
   if (page != NULL)
     gtk_widget_grab_focus (GTK_WIDGET (page));
 }
 
-static void
-group_header_func (GtkListBoxRow *row,
-                   GtkListBoxRow *before,
-                   gpointer       user_data)
-{
-  IdeSearchPopoverGroup *before_group;
-
-  if (before == NULL)
-    return;
-
-  if ((before_group = g_object_get_data (G_OBJECT (before), "GROUP")) &&
-      ide_str_equal (("_Everything"), ide_search_popover_group_get_title (before_group)))
-    gtk_list_box_row_set_header (row,
-                                 g_object_new (GTK_TYPE_SEPARATOR,
-                                               "orientation", GTK_ORIENTATION_HORIZONTAL,
-                                               NULL));
-}
-
-static GtkWidget *
-create_group_row (gpointer item,
-                  gpointer user_data)
-{
-  IdeSearchPopoverGroup *group = item;
-  const char *title;
-  const char *icon_name;
-  GtkWidget *box;
-  GtkWidget *image;
-  GtkWidget *label;
-  GtkWidget *row;
-
-  g_assert (IDE_IS_MAIN_THREAD ());
-  g_assert (IDE_IS_SEARCH_POPOVER_GROUP (group));
-
-  title = ide_search_popover_group_get_title (group);
-  icon_name = ide_search_popover_group_get_icon_name (group);
-
-  box = g_object_new (GTK_TYPE_BOX,
-                      "orientation", GTK_ORIENTATION_HORIZONTAL,
-                      "spacing", 9,
-                      NULL);
-  image = g_object_new (GTK_TYPE_IMAGE,
-                        "icon-name", icon_name,
-                        NULL);
-  label = g_object_new (GTK_TYPE_LABEL,
-                        "label", title,
-                        "xalign", .0f,
-                        "ellipsize", PANGO_ELLIPSIZE_END,
-                        "use-underline", TRUE,
-                        NULL);
-  gtk_box_append (GTK_BOX (box), image);
-  gtk_box_append (GTK_BOX (box), label);
-
-  row = g_object_new (GTK_TYPE_LIST_BOX_ROW,
-                      "css-classes", IDE_STRV_INIT ("sidebar-row"),
-                      "child", box,
-                      NULL);
-
-  g_object_set_data_full (G_OBJECT (row),
-                          "GROUP",
-                          g_object_ref (group),
-                          g_object_unref);
-
-  return GTK_WIDGET (row);
-}
-
 static void
 ide_search_popover_set_search_engine (IdeSearchPopover *self,
                                       IdeSearchEngine  *search_engine)
@@ -270,6 +191,8 @@ ide_search_popover_search_cb (GObject      *object,
 
   gtk_single_selection_set_model (self->selection, G_LIST_MODEL (results));
 
+  gtk_stack_set_visible_child_name (self->stack, "results");
+
   ide_search_popover_after_search (self);
 
   IDE_EXIT;
@@ -278,11 +201,9 @@ ide_search_popover_search_cb (GObject      *object,
 static gboolean
 ide_search_popover_search_source_func (gpointer data)
 {
-  IdeSearchCategory category = IDE_SEARCH_CATEGORY_EVERYTHING;
-  IdeSearchPopoverGroup *group;
+  g_autofree char *query_stripped = NULL;
+  IdeSearchCategory category;
   IdeSearchPopover *self = data;
-  GtkListBoxRow *row;
-  GListModel *model;
   const char *query;
 
   IDE_ENTRY;
@@ -297,19 +218,54 @@ ide_search_popover_search_source_func (gpointer data)
 
   ide_search_popover_cancel (self);
 
-  query = gtk_editable_get_text (GTK_EDITABLE (self->entry));
+  query = gtk_editable_get_text (GTK_EDITABLE (self->text));
 
   if (ide_str_empty0 (query))
     IDE_GOTO (failure);
 
-  /* Get the category for the query */
-  if ((row = gtk_list_box_get_selected_row (self->providers_list_box)) &&
-      (group = g_object_get_data (G_OBJECT (row), "GROUP")) &&
-      IDE_IS_SEARCH_POPOVER_GROUP (group))
-    category = ide_search_popover_group_get_category (group);
+  switch (*query)
+    {
+    case '?':
+      category = IDE_SEARCH_CATEGORY_DOCUMENTATION;
+      query++;
+      break;
 
-  /* Fast path to just filter our previous result set */
+    case '@':
+      category = IDE_SEARCH_CATEGORY_SYMBOLS;
+      query++;
+      break;
+
+    case '>':
+      /* TODO: Maybe commands here too? */
+      category = IDE_SEARCH_CATEGORY_ACTIONS;
+      query++;
+      break;
+
+    case '~':
+      category = IDE_SEARCH_CATEGORY_FILES;
+      query++;
+      break;
+
+    default:
+      category = IDE_SEARCH_CATEGORY_EVERYTHING;
+      break;
+    }
+
+  query = query_stripped = g_strstrip (g_strdup (query));
+
+  if (ide_str_empty0 (query))
+    IDE_GOTO (failure);
+
+#if 0
+  /* Drop refiltering because the performance is extremely bad for
+   * lazy loaded search results. Instead, it would be better to
+   * allow for the provider to refilter their listmodel like I
+   * did in GtkSourceCompletion.
+   */
+
+  GListModel *model;
   if (category == self->last_category &&
+      !ide_str_empty0 (query) &&
       (model = gtk_single_selection_get_model (self->selection)) &&
       IDE_IS_SEARCH_RESULTS (model) &&
       ide_search_results_refilter (IDE_SEARCH_RESULTS (model), query))
@@ -317,6 +273,7 @@ ide_search_popover_search_source_func (gpointer data)
       ide_search_popover_after_search (self);
       IDE_RETURN (G_SOURCE_REMOVE);
     }
+#endif
 
   self->last_category = category;
 
@@ -333,6 +290,7 @@ ide_search_popover_search_source_func (gpointer data)
 failure:
   self->activate_after_search = FALSE;
   gtk_single_selection_set_model (self->selection, NULL);
+  gtk_stack_set_visible_child_name (self->stack, "empty");
 
   IDE_RETURN (G_SOURCE_REMOVE);
 }
@@ -351,7 +309,7 @@ ide_search_popover_queue_search (IdeSearchPopover *self)
 
   g_clear_handle_id (&self->queued_search, g_source_remove);
 
-  text = gtk_editable_get_text (GTK_EDITABLE (self->entry));
+  text = gtk_editable_get_text (GTK_EDITABLE (self->text));
 
   if (self->activate_after_search)
     delay = 0;
@@ -426,8 +384,8 @@ ide_search_popover_search_focus (GtkWidget  *widget,
 {
   IdeSearchPopover *self = IDE_SEARCH_POPOVER (widget);
 
-  gtk_widget_grab_focus (GTK_WIDGET (self->entry));
-  gtk_editable_select_region (GTK_EDITABLE (self->entry), 0, -1);
+  gtk_widget_grab_focus (GTK_WIDGET (self->text));
+  gtk_editable_select_region (GTK_EDITABLE (self->text), 0, -1);
 }
 
 static void
@@ -437,16 +395,20 @@ ide_search_popover_move_action (GtkWidget  *widget,
 {
   IdeSearchPopover *self = (IdeSearchPopover *)widget;
   guint selected;
-  int dir;
+  int dir = 1;
 
   IDE_ENTRY;
 
   g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (IDE_IS_SEARCH_POPOVER (self));
-  g_assert (g_variant_is_of_type (param, G_VARIANT_TYPE_INT32));
+  g_assert (!param || g_variant_is_of_type (param, G_VARIANT_TYPE_INT32));
+
+  if (param != NULL)
+    dir = g_variant_get_int32 (param);
+  else if (ide_str_equal0 (action_name, "search.move-up"))
+    dir = -1;
 
   selected = gtk_single_selection_get_selected (self->selection);
-  dir = g_variant_get_int32 (param);
 
   if (dir < 0)
     {
@@ -461,47 +423,24 @@ ide_search_popover_move_action (GtkWidget  *widget,
     }
 
   if (selected < g_list_model_get_n_items (G_LIST_MODEL (self->selection)))
-    gtk_single_selection_set_selected (self->selection, selected);
+    {
+      GtkScrollInfo *scroll_info;
+
+      gtk_single_selection_set_selected (self->selection, selected);
+
+      scroll_info = gtk_scroll_info_new ();
+      gtk_scroll_info_set_enable_vertical (scroll_info, TRUE);
+
+      gtk_list_view_scroll_to (self->list_view, selected, 0, scroll_info);
+    }
 
   IDE_EXIT;
 }
 
-static void
-ide_search_popover_next_match_cb (IdeSearchPopover *self,
-                                  GtkSearchEntry   *entry)
-{
-  guint selected;
-
-  g_assert (IDE_IS_SEARCH_POPOVER (self));
-  g_assert (GTK_IS_SEARCH_ENTRY (entry));
-
-  selected = gtk_single_selection_get_selected (self->selection);
-
-  if (selected + 1 < g_list_model_get_n_items (G_LIST_MODEL (self->selection)))
-    gtk_single_selection_set_selected (self->selection, selected + 1);
-}
-
-static void
-ide_search_popover_previous_match_cb (IdeSearchPopover *self,
-                                      GtkSearchEntry   *entry)
-{
-  guint selected;
-
-  g_assert (IDE_IS_SEARCH_POPOVER (self));
-  g_assert (GTK_IS_SEARCH_ENTRY (entry));
-
-  selected = gtk_single_selection_get_selected (self->selection);
-
-  if (selected > 0)
-    gtk_single_selection_set_selected (self->selection, selected - 1);
-}
-
 static void
 ide_search_popover_set_preview (IdeSearchPopover *self,
                                 IdeSearchPreview *preview)
 {
-  const char *title = NULL;
-  const char *subtitle = NULL;
   gboolean can_show_preview;
 
   IDE_ENTRY;
@@ -514,18 +453,7 @@ ide_search_popover_set_preview (IdeSearchPopover *self,
   adw_bin_set_child (self->preview_bin, GTK_WIDGET (preview));
   can_show_preview = self->has_preview && self->show_preview;
 
-  gtk_revealer_set_reveal_child (self->preview_revealer, can_show_preview);
-
-  if (preview != NULL)
-    {
-      title = ide_search_preview_get_title (preview);
-      subtitle = ide_search_preview_get_subtitle (preview);
-    }
-
-  /* TODO: We might want to bind these properties */
-
-  adw_window_title_set_title (self->preview_title, title);
-  adw_window_title_set_subtitle (self->preview_title, subtitle);
+  gtk_widget_set_visible (GTK_WIDGET (self->preview_bin), can_show_preview);
 
   IDE_EXIT;
 }
@@ -572,8 +500,8 @@ ide_search_popover_show (GtkWidget *widget)
 
   GTK_WIDGET_CLASS (ide_search_popover_parent_class)->show (widget);
 
-  gtk_widget_grab_focus (GTK_WIDGET (self->entry));
-  gtk_editable_select_region (GTK_EDITABLE (self->entry), 0, -1);
+  gtk_widget_grab_focus (GTK_WIDGET (self->text));
+  gtk_editable_select_region (GTK_EDITABLE (self->text), 0, -1);
 }
 
 static gboolean
@@ -583,7 +511,7 @@ ide_search_popover_grab_focus (GtkWidget *widget)
 
   g_assert (IDE_IS_SEARCH_POPOVER (self));
 
-  return gtk_widget_grab_focus (GTK_WIDGET (self->entry));
+  return gtk_widget_grab_focus (GTK_WIDGET (self->text));
 }
 
 static void
@@ -597,7 +525,6 @@ ide_search_popover_dispose (GObject *object)
 
   g_clear_object (&self->cancellable);
   g_clear_object (&self->search_engine);
-  g_clear_object (&self->groups);
 
   G_OBJECT_CLASS (ide_search_popover_parent_class)->dispose (object);
 }
@@ -678,53 +605,34 @@ ide_search_popover_class_init (IdeSearchPopoverClass *klass)
   g_resources_register (ide_search_get_resource ());
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/libide-gui/ui/ide-search-popover.ui");
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, center);
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, entry);
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, left);
   gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, list_view);
   gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, preview_bin);
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, preview_revealer);
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, preview_toggle);
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, preview_title);
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, providers_list_box);
-  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, right);
   gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, selection);
+  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, stack);
+  gtk_widget_class_bind_template_child (widget_class, IdeSearchPopover, text);
   gtk_widget_class_bind_template_callback (widget_class, ide_search_popover_activate_cb);
   gtk_widget_class_bind_template_callback (widget_class, ide_search_popover_category_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, ide_search_popover_entry_activate_cb);
   gtk_widget_class_bind_template_callback (widget_class, ide_search_popover_search_changed_cb);
-  gtk_widget_class_bind_template_callback (widget_class, ide_search_popover_next_match_cb);
-  gtk_widget_class_bind_template_callback (widget_class, ide_search_popover_previous_match_cb);
   gtk_widget_class_bind_template_callback (widget_class, ide_search_popover_selection_changed_cb);
 
   gtk_widget_class_install_action (widget_class, "search.hide", NULL, ide_search_popover_hide_action);
   gtk_widget_class_install_action (widget_class, "search.move", "i", ide_search_popover_move_action);
+  gtk_widget_class_install_action (widget_class, "search.move-up", NULL, ide_search_popover_move_action);
+  gtk_widget_class_install_action (widget_class, "search.move-down", NULL, ide_search_popover_move_action);
   gtk_widget_class_install_action (widget_class, "search.focus", NULL, ide_search_popover_search_focus);
 
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_k, GDK_CONTROL_MASK, "search.focus", NULL);
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_l, GDK_CONTROL_MASK, "search.focus", NULL);
-
-  g_type_ensure (IDE_TYPE_SEARCH_POPOVER_GROUP);
 }
 
 static void
 ide_search_popover_init (IdeSearchPopover *self)
 {
   self->show_preview = TRUE;
-  self->groups = g_list_store_new (IDE_TYPE_SEARCH_POPOVER_GROUP);
   self->settings = g_settings_new ("org.gnome.builder");
 
   gtk_widget_init_template (GTK_WIDGET (self));
-
-  gtk_list_box_set_header_func (self->providers_list_box,
-                                group_header_func, self, NULL);
-  gtk_list_box_bind_model (self->providers_list_box,
-                           G_LIST_MODEL (self->groups),
-                           create_group_row, self, NULL);
-
-  g_settings_bind (self->settings, "preview-search-results",
-                   self->preview_toggle, "active",
-                   G_SETTINGS_BIND_DEFAULT);
 }
 
 GtkWidget *
@@ -737,23 +645,6 @@ ide_search_popover_new (IdeSearchEngine *search_engine)
                        NULL);
 }
 
-void
-ide_search_popover_present (IdeSearchPopover *self,
-                            int               parent_width,
-                            int               parent_height)
-{
-  GtkRequisition left, right;
-
-  g_return_if_fail (IDE_IS_SEARCH_POPOVER (self));
-
-  gtk_widget_get_preferred_size (GTK_WIDGET (self->left), &left, NULL);
-  gtk_widget_get_preferred_size (GTK_WIDGET (self->preview_revealer), &right, NULL);
-
-  gtk_popover_set_offset (GTK_POPOVER (self), -(left.width/2) + (right.width/2), 0);
-  gtk_popover_set_pointing_to (GTK_POPOVER (self), &(GdkRectangle) { parent_width/2, 100, 1, 1 });
-  gtk_popover_present (GTK_POPOVER (self));
-}
-
 gboolean
 ide_search_popover_get_show_preview (IdeSearchPopover *self)
 {
@@ -778,24 +669,3 @@ ide_search_popover_set_show_preview (IdeSearchPopover *self,
       ide_search_popover_selection_changed_cb (self, NULL, self->selection);
     }
 }
-
-static void
-ide_search_popover_add_child (GtkBuildable *buildable,
-                              GtkBuilder   *builder,
-                              GObject      *object,
-                              const char   *type)
-{
-  IdeSearchPopover *self = IDE_SEARCH_POPOVER (buildable);
-
-  if (IDE_IS_SEARCH_POPOVER_GROUP (object))
-    g_list_store_append (self->groups, object);
-  else
-    parent_buildable_iface->add_child (buildable, builder, object, type);
-}
-
-static void
-buildable_iface_init (GtkBuildableIface *iface)
-{
-  parent_buildable_iface = g_type_interface_peek_parent (iface);
-  iface->add_child = ide_search_popover_add_child;
-}
